@@ -334,7 +334,7 @@ gst_identity_do_sync (GstIdentity * identity, GstClockTime running_time)
       return GST_FLOW_FLUSHING;
     }
 
-    while (identity->blocked)
+    while (identity->blocked && !identity->flushing)
       g_cond_wait (&identity->blocked_cond, GST_OBJECT_GET_LOCK (identity));
 
     if (identity->flushing) {
@@ -464,6 +464,7 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
     if (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_START) {
       GST_OBJECT_LOCK (identity);
       identity->flushing = TRUE;
+      g_cond_signal (&identity->blocked_cond);
       if (identity->clock_id) {
         GST_DEBUG_OBJECT (identity, "unlock clock wait");
         gst_clock_id_unschedule (identity->clock_id);
@@ -612,6 +613,7 @@ gst_identity_update_last_message_for_buffer (GstIdentity * identity,
       GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET_END (buf),
       GST_BUFFER_FLAGS (buf), flag_str, meta_str ? meta_str : "none", buf);
   g_free (flag_str);
+  g_free (meta_str);
 
   GST_OBJECT_UNLOCK (identity);
 
@@ -1011,7 +1013,7 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       GST_OBJECT_LOCK (identity);
       identity->blocked = FALSE;
-      g_cond_broadcast (&identity->blocked_cond);
+      g_cond_signal (&identity->blocked_cond);
       GST_OBJECT_UNLOCK (identity);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -1022,7 +1024,7 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
         gst_clock_id_unschedule (identity->clock_id);
       }
       identity->blocked = FALSE;
-      g_cond_broadcast (&identity->blocked_cond);
+      g_cond_signal (&identity->blocked_cond);
       GST_OBJECT_UNLOCK (identity);
       break;
     default:
